/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Configuration.java 20821 2011-02-02 16:59:06Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.configuration;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * Create a new Configuration object.
 * @author skabore
 */
public class Configuration {

    /**
     * The jrmp port number.
     */
    private static final String JRMP_PORT = "1099";

    /**
     * The irmi port number.
     */
    private static final String IRMI_PORT = "1098";

    /**
     * The iiop port number.
     */
    private static final String IIOP_PORT = "2001";

    /**
     * The lmi port number.
     */
    private static final String LMI_PORT = "0";

    /**
     * The web container.
     */
    // private String webContainer;

    /**
     * The db name value.
     */
    private String dbName;

    /**
     * The db port value.
     */
    private String dbPort;

    /**
     * The db user value.
     */
    private String dbUser;

    /**
     * The db user password value.
     */
    private String dbUserPassword;

    /**
     * The http port value.
     */
    private String httpPort;

    /**
     * The redirect port value.
     */
    private String redirectPort;

    /**
     * The httpsConnectorActivation.
     */
    private boolean httpsConnectorActivation;

    /**
     * The jvmRouteActivation.
     */
    private boolean jvmRouteActivation;

    /**
     * The jkPortActivation.
     */
    private boolean jkPortActivation;

    /**
     * The list of protocols.
     */
    private List<Protocol> protocols;

    /**
     * The list of services.
     */
    private List<Service> services;

    /**
     * The list of JDBCDatasource.
     */
    private List<JDBCDatasource> jdbcDatasource;

    /**
     * The jms.
     */
    private Jms jms;

    /**
     * Default constructor.
     */
    public Configuration() {
        this.services = new ArrayList<Service>();
        this.protocols = new ArrayList<Protocol>();
        this.jdbcDatasource = new ArrayList<JDBCDatasource>();
        this.jms = new Jms();
    }

    /**
     * Add a new protocol to the list of protocols.
     * @param protocol the protocol to add
     */
    public void addProtocols(final Protocol protocol) {
        if (!this.protocols.contains(protocol)) {
            this.protocols.add(protocol);
        }
    }

    /**
     * Remove a protocol from the list of protocols.
     * @param protocol the protocol to remove
     */
    public void removeProtocols(final Protocol protocol) {
        this.protocols.remove(protocol);
    }

    /**
     * Remove the protocols lis.
     */
    public void removeAllProtocols() {
        for (Protocol p : this.protocols) {
            this.protocols.remove(p);
        }
    }

    /**
     * Add a new service to the list of services.
     * @param srv the service to add
     */
    public void addService(final Service srv) {
        if (!this.services.contains(srv)) {
            this.services.add(srv);
        }
    }

    /**
     * Remove a service from the list of services.
     * @param srv the service to remove
     */
    public void removeService(final Service srv) {
        this.services.remove(srv);
    }

    /**
     * @return httpsConnectorActivation
     */
    public boolean getHttpsConnectorActivation() {
        return this.httpsConnectorActivation;
    }

    /**
     * Set the httpsConnectorActivation.
     * @param httpsConnectorActivation the httpsConnectorActivation new value
     */
    public void setHttpsConnectorActivation(final boolean httpsConnectorActivation) {
        this.httpsConnectorActivation = httpsConnectorActivation;
    }

    /**
     * @return httpPort
     */
    public String getHttpPort() {
        return this.httpPort;
    }

    /**
     * Set the httpPort.
     * @param httpPort the httpPort new value
     */
    public void setHttpPort(final String httpPort) {
        this.httpPort = httpPort;
    }

    /**
     * @return jkPortActivation
     */
    public boolean getJkPortActivation() {
        return this.jkPortActivation;
    }

    /**
     * Set the jkPortActivation.
     * @param jkPortActivation the jkPortActivation new value
     */
    public void setJkPortActivation(final boolean jkPortActivation) {
        this.jkPortActivation = jkPortActivation;
    }

    /**
     * @return jvmRouteActivation
     */
    public boolean getJvmRouteActivation() {
        return this.jvmRouteActivation;
    }

    /**
     * Set the jvmRouteActivation.
     * @param jvmRouteActivation the jvmRouteActivation new value
     */
    public void setJvmRouteActivation(final boolean jvmRouteActivation) {
        this.jvmRouteActivation = jvmRouteActivation;
    }

    /**
     * @return protocols
     */
    public List<Protocol> getProtocols() {
        return this.protocols;
    }

    /**
     * @return protocol.
     * @param protocolName the name of the selected protocol
     */
    public Protocol getProtocolByName(final String protocolName) {
        for (Protocol protocol : this.protocols) {
            if (protocol.getName().equalsIgnoreCase(protocolName)) {
                return protocol;
            }
        }
        return null;
    }

    /**
     * Set the protocols.
     * @param protocols the protocols new value
     */
    public void setProtocols(final List<Protocol> protocols) {
        this.protocols = protocols;
    }

    /**
     * @return redirectPort
     */
    public String getRedirectPort() {
        return this.redirectPort;
    }

    /**
     * Set the redirectPort.
     * @param redirectPort the redirectPort new value
     */
    public void setRedirectPort(final String redirectPort) {
        this.redirectPort = redirectPort;
    }

    /**
     * @return services
     */
    public List<Service> getServices() {
        return this.services;
    }

    /**
     * @return service name.
     * @param srvName the name of the selected service
     */
    public Service getServiceByName(final String srvName) {
        for (Service srv : this.services) {
            if (srv.getName().equalsIgnoreCase(srvName)) {
                return srv;
            }
        }
        return null;
    }

    /**
     * Set the services.
     * @param srv the services new value
     */
    public void setServices(final List<Service> srv) {
        this.services = srv;
    }

    /**
     * @return webContainer
     */
    /*
     * public String getWebContainer() { return webContainer; }
     */

    /**
     * Set the webContainer.
     * @param webContainer the webContainer new value
     */
    /*
     * public void setWebContainer(final String webContainer) {
     * this.webContainer = webContainer; }
     */

    /**
     * @return dbName
     */
    public String getDbName() {
        return this.dbName;
    }

    /**
     * Set the dbName.
     * @param dbName new value
     */
    public void setDbName(final String dbName) {
        this.dbName = dbName;
    }

    /**
     * @return dbPort
     */
    public String getDbPort() {
        return this.dbPort;
    }

    /**
     * Set the dbPort.
     * @param dbPort new value
     */
    public void setDbPort(final String dbPort) {
        this.dbPort = dbPort;
    }

    /**
     * @return dbUser
     */
    public String getDbUser() {
        return this.dbUser;
    }

    /**
     * Set the dbUser.
     * @param dbUser new value
     */
    public void setDbUser(final String dbUser) {
        this.dbUser = dbUser;
    }

    /**
     * @return dbUserPassword
     */
    public String getDbUserPassword() {
        return this.dbUserPassword;
    }

    /**
     * Set the dbUserPassword.
     * @param dbUserPassword new value
     */
    public void setDbUserPassword(final String dbUserPassword) {
        this.dbUserPassword = dbUserPassword;
    }

    /**
     * @return the jdbcDatasource.
     */
    public List<JDBCDatasource> getJdbcDatasource() {
        return this.jdbcDatasource;
    }

    /**
     * Set the jdbcDatasource value.
     * @param jdbcDatasource new value
     */
    public void setJdbcDatasource(final List<JDBCDatasource> jdbcDatasource) {
        this.jdbcDatasource = jdbcDatasource;
    }

    /**
     * @return jms.
     */
    public Jms getJms() {
        return this.jms;
    }

    /**
     * Set the jms.
     * @param jms the new jms value
     */
    public void setJms(final Jms jms) {
        this.jms = jms;
    }

    /**
     * @return jdbcDatasource.
     * @param jdbcName the name of the selected jdbc
     */
    public JDBCDatasource getJdbcDatasourceByName(final String jdbcName) {
        for (JDBCDatasource jdbc : this.jdbcDatasource) {
            if (jdbc.getJdbcConfiguration().jndiName.equalsIgnoreCase(jdbcName)) {
                return jdbc;
            }
        }
        return null;
    }

    /**
     * Add a new jdbcDatasource to the list of jdbcDatasource.
     * @param jdbc the jdbc to add
     */
    public void addJdbcDatasource(final JDBCDatasource jdbc) {
        if (!this.jdbcDatasource.contains(jdbc)) {
            this.jdbcDatasource.add(jdbc);
        }
    }

    /**
     * Remove a jdbc datasource from the list of jdbcDatasource.
     * @param datasource the jdbc datasource to remove
     */
    public void removeJdbcDatasource(final JDBCDatasource datasource) {
        this.jdbcDatasource.remove(datasource);
    }

    /**
     * Initialize the configuration by the default value.
     */
    public void initialize() {
        // webContainer = "tomcat";
        this.dbName = "db_jonas";
        this.dbPort = "9001";
        this.dbUser = "jonas";
        this.dbUserPassword = "jonas";
        this.httpPort = "9000";
        this.redirectPort = "0";
        this.httpsConnectorActivation = false;
        this.jkPortActivation = false;
        this.jvmRouteActivation = false;
        initializeServices();
        initializeProtocoles();
        // initializeJdbcDatasource();
        initializeJms();
    }

    /**
     * Initialize services by the default value.
     */
    private void initializeServices() {
        Service jtm = new Service("jtm", false);
        Service db = new Service("db", false);
        Service dbm = new Service("dbm", false);
        Service security = new Service("security", false);
        Service resource = new Service("resource", false);
        Service maildb = new Service("mail", false);
        Service cmi = new Service("cmi", false);
        Service ha = new Service("ha", false);

        Service versioning = new Service("versioning", false);
        Service ejb2 = new Service("ejb2", false);
        Service ejb3 = new Service("ejb3", false);
        Service jaxrpc = new Service("jaxrpc", false);
        Service jaxws = new Service("jaxws", false);
        Service web = new Service("web", false);
        Service ear = new Service("ear", false);
        Service depmonitor = new Service("depmonitor", true);

        Service discovery = new Service("discovery", false);
        Service resourcemonitor = new Service("resourcemonitor", false);
        Service smartclient = new Service("smartclient", false);
        Service audit = new Service("audit", false);
        Service cdi = new Service("cdi", false);

        addService(jtm);
        addService(db);
        addService(dbm);
        addService(security);
        addService(resource);
        addService(maildb);
        addService(cmi);
        addService(ha);
        addService(versioning);
        addService(ejb2);
        addService(ejb3);
        addService(jaxrpc);
        addService(jaxws);
        addService(web);
        addService(ear);
        addService(depmonitor);
        addService(discovery);
        addService(resourcemonitor);
        addService(smartclient);
        addService(audit);
        addService(cdi);
    }

    /**
     * Initialize protocoles by the default value.
     */
    private void initializeProtocoles() {
        Protocol jrmp = new Protocol("jrmp", JRMP_PORT, true);
        Protocol iiop = new Protocol("iiop", IIOP_PORT, false);
        Protocol irmi = new Protocol("irmi", IRMI_PORT, false);
        Protocol lmi = new Protocol("lmi", LMI_PORT, false);
        addProtocols(jrmp);
        addProtocols(iiop);
        addProtocols(irmi);
        addProtocols(lmi);
    }

    /**
     * Initialize JdbcDatasource by the default value.
     */
    private void initializeJdbcDatasource() {
        JDBCDatasource jdbc = new JDBCDatasource();
        jdbc.setDatasourceName("HSQL1");
        jdbc.setDbmsName("HSQL");
        jdbc.setRaName("hsqlRA.rar");
        jdbc.initializeJdbcConfiguration();
        addJdbcDatasource(jdbc);
    }

    /**
     * Initialize jms by the default value.
     */
    private void initializeJms() {
        this.jms.setPort("16010");
        List<String> queues = new ArrayList<String>(Arrays.asList("sampleQueue"));
        this.jms.setQueues(queues);
        List<String> topics = new ArrayList<String>(Arrays.asList("sampleTopic"));
        this.jms.setTopics(topics);
    }
}
