/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Configurator.java 20828 2011-02-03 10:12:20Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.configurator;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.tools.ant.taskdefs.Move;
import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.configuration.JDBCDatasource;
import org.ow2.jonas.autostart.utility.Utility;
import org.ow2.jonas.tools.configurator.Jonas;
import org.ow2.jonas.tools.configurator.api.JDBCConfiguration;
import org.ow2.jonas.tools.configurator.api.JonasConfigurator;

/**
 * This class use the JOnAS configurator sub-project to configure a
 * fresh-installed JOnAS server.
 * @author mathieu ancelin
 */
public class Configurator {

    /**
     * The path of the JONAS_ROOT.
     */
    private String jonasRoot = null;

    /**
     * The path of the JONAS_BASE.
     */
    private String jonasBase = null;

    /**
     * The JAVA_HOME value.
     */
    private String javaHome = null;

    /**
     * The actual JOnAS server on the machine identified by its JONAS_ROOT.
     */
    private Jonas unconfiguredJonas = null;

    /**
     * The actual configurator for the JOnAS server.
     */
    private JonasConfigurator configurator = null;

    /**
     * The configuration.
     */
    private Configuration configuration;

    /**
     * The file separator.
     */
    private static final String FILE_SEPARATOR = System.getProperty("file.separator");

    /**
     * The logger for traces.
     */
    private static final Logger logger = Logger.getLogger(Configurator.class.getName());

    /**
     * The temporary working directory.
     */
    private File workdirectory = null;

    /**
     * The jdbc bundle file.
     */
    private List<File> jdbcBundles = null;

    /**
     * Constructor.
     * @param jonasRoot the path of the JONAS_ROOT
     * @param jonasBase the path of the JONAS_BASE
     * @param configuration the configuration object
     */
    public Configurator(final String jonasRoot, final String jonasBase, final Configuration configuration, final File workdirectory) {
        this.jonasRoot = jonasRoot;
        this.jonasBase = jonasBase;
        this.javaHome = System.getenv("JAVA_HOME");
        this.configuration = configuration;
        this.workdirectory = workdirectory;
        this.jdbcBundles = new ArrayList<File>();
    }

    /**
     * Apply configuration.
     */
    public void doConfiguration() {
        if (Utility.isValidJOnASRootFolder(new File(this.jonasRoot))) {

            this.unconfiguredJonas = new Jonas(this.jonasRoot);
            this.configurator = this.unconfiguredJonas.getJonasConfigurator();
            this.configurator.setJonasBase(this.jonasBase);

            // Services
            this.configurator.setServices(getServices());

            // Protocols
            if (isValidProtocolsParams()) {
                this.configurator.setProtocolsList(getProtocolsList());
                this.configurator.setProtocolsJrmpPort(getJrmpPort());
                this.configurator.setProtocolsIiopPort(getIiopPort());
                this.configurator.setProtocolsIrmiPort(getIrmiPort());
            }

            // Hsql DB
            if (isValidDbParams()) {
                this.configurator.setDbName(getDbName());
                this.configurator.setDbPort(getDbPort());
                this.configurator.setDbUsers(getDbUsers());
                // configurator.set(getDbUserPassword());
            }

            // Jms
            if (isValidJmsParams()) {
                this.configurator.setJmsPort(getJmsPort());
                this.configurator.setJmsQueues(getJmsQueues());
                this.configurator.setJmsTopics(getJmsTopics());
            }

            // Http(s) protocol
            if (isValidHttpParams()) {
                this.configurator.setHttpPort(getHttpPort());
                this.configurator.setHttpsConnectorActivation(getHttpsActivation());
                this.configurator.setJvmRoute(getJvmRouteActivation());
            }

            // Datasource
            if (isValidJdbcConfigParams()) {
                JDBCConfiguration jdbcConfiguration = new JDBCConfiguration();
                for (JDBCDatasource datasource : this.configuration.getJdbcDatasource()) {
                    jdbcConfiguration = datasource.getJdbcConfiguration();
                    this.configurator.addJdbcRA(datasource.getRaName(), jdbcConfiguration);
                    createJdbcOsgiBundle(datasource.getDatasourceName(), jdbcConfiguration.driverName);
                }
            }
            this.configurator.execute();

            // Deploy the bundle in JOnAS_Base/deploy directory
            if (isValidJdbcConfigParams()) {
                Move mv = new Move();
                for (File bnd : this.jdbcBundles) {
                    mv.setFile(bnd);
                    mv.setTodir(new File(this.jonasBase, "deploy"));
                    mv.execute();
                }
            }
        }
    }

    /**
     * @param datasource
     * @param driverJarFile
     */
    private void createJdbcOsgiBundle(final String datasource, final String driverJarFile) {
        Runtime runtime = Runtime.getRuntime();
        String javaDir = this.javaHome + FILE_SEPARATOR + "bin" + FILE_SEPARATOR + "java";
        File javaExec = new File(javaDir);
        javaExec.setExecutable(true, false);

        File bundle = new File(this.workdirectory, datasource + ".jar");
        // FIXME: Use inline BND jar file
        String bnd = "";
        String[] env = {"JAVA_HOME=" + this.javaHome};
        String[] command = {javaExec.getAbsolutePath(), "-jar", bnd, "-failok", "wrap", "-output", bundle.getName(),
                driverJarFile};
        if (javaExec.exists() && new File(bnd).exists()) {
            try {
                runtime.exec(command, env, this.workdirectory);
                this.jdbcBundles.add(bundle);
            } catch (IOException ex) {
                logger.log(Level.INFO, ex.getMessage());
            }
        }/*
          * else { ; }
          */
    }

    /**
     * @return the list of services.
     */
    private String getServices() {
        String service = "";
        if (this.configuration.getServices() != null) {
            for (int i = 0; i < this.configuration.getServices().size(); i++) {
                if (this.configuration.getServices().get(i).getIsEnabled()) {
                    service = service.equals("") ? this.configuration.getServices().get(i).getName() : service + ","
                            + this.configuration.getServices().get(i).getName();
                }
            }
        }
        return service;
    }

    /**
     * @return the list of protocols.
     */
    private String getProtocolsList() {
        String protocols = "";
        if (this.configuration.getProtocols() != null) {
            for (int i = 0; i < this.configuration.getProtocols().size(); i++) {
                if (this.configuration.getProtocols().get(i).getIsEnabled()) {
                    protocols = protocols.equals("") ? this.configuration.getProtocols().get(i).getName() : protocols + ","
                            + this.configuration.getProtocols().get(i).getName();
                }
            }
        }
        return protocols;
    }

    /**
     * @return jrmp port.
     */
    private String getJrmpPort() {
        String port = "1099";
        if (this.configuration.getProtocols() != null) {
            for (int i = 0; i < this.configuration.getProtocols().size(); i++) {
                if (this.configuration.getProtocols().get(i).getName().equals("jrmp")) {
                    return this.configuration.getProtocols().get(i).getPort();
                }
            }
        }
        return port;
    }

    /**
     * @return iiop port.
     */
    private String getIiopPort() {
        String port = "2001";
        if (this.configuration.getProtocols() != null) {
            for (int i = 0; i < this.configuration.getProtocols().size(); i++) {
                if (this.configuration.getProtocols().get(i).getName().equals("iiop")) {
                    return this.configuration.getProtocols().get(i).getPort();
                }
            }
        }
        return port;
    }

    /**
     * @return irmi port.
     */
    private String getIrmiPort() {
        String port = "1098";
        if (this.configuration.getProtocols() != null) {
            for (int i = 0; i < this.configuration.getProtocols().size(); i++) {
                if (this.configuration.getProtocols().get(i).getName().equals("irmi")) {
                    return this.configuration.getProtocols().get(i).getPort();
                }
            }
        }
        return port;
    }

    /**
     * @return lmi port.
     */
    private String getLmiPort() {
        String port = "0";
        if (this.configuration.getProtocols() != null) {
            for (int i = 0; i < this.configuration.getProtocols().size(); i++) {
                if (this.configuration.getProtocols().get(i).getName().equals("lmi")) {
                    return this.configuration.getProtocols().get(i).getPort();
                }
            }
        }
        return port;
    }

    /**
     * @return jms port.
     */
    private String getJmsPort() {
        String port = "0";
        if (this.configuration.getJms() != null) {
            port = this.configuration.getJms().getPort();
        }
        return port;
    }

    /**
     * @return jms queues.
     */
    private String getJmsQueues() {
        String queues = "";
        if (this.configuration.getJms() != null) {
            for (String queue : this.configuration.getJms().getQueues()) {
                if (queues.equals("")) {
                    queues = queue;
                } else {
                    queues = queues + "," + queue;
                }
            }
        }
        return queues;
    }

    /**
     * @return jms topics.
     */
    private String getJmsTopics() {
        String topics = "";
        if (this.configuration.getJms() != null) {
            for (String topic : this.configuration.getJms().getTopics()) {
                if (topics.equals("")) {
                    topics = topic;
                } else {
                    topics = topics + "," + topic;
                }
            }
        }
        return topics;
    }

    /**
     * @return http port.
     */
    private String getHttpPort() {
        return this.configuration.getHttpPort();
    }

    /**
     * @return https activation.
     */
    private Boolean getHttpsActivation() {
        return Boolean.valueOf(this.configuration.getHttpsConnectorActivation());
    }

    /**
     * @return JvmRoute activation.
     */
    private String getJvmRouteActivation() {
        return this.configuration.getJvmRouteActivation() + "";
    }

    /**
     * @return JkPort activation.
     */
    private String getJkPortActivation() {
        return this.configuration.getJkPortActivation() + "";
    }

    /**
     * @return db port.
     */
    private String getDbPort() {
        return this.configuration.getDbPort();
    }

    /**
     * @return db name.
     */
    private String getDbName() {
        return this.configuration.getDbName();
    }

    /**
     * @return db user name.
     */
    private String getDbUsers() {
        return this.configuration.getDbUser();
    }

    /**
     * @return db user password.
     */
    private String getDbUserPassword() {
        return this.configuration.getDbUserPassword();
    }

    /**
     * Set JONAS_ROOT value.
     * @param jonasRoot the JONAS_ROOT value
     */
    public void setJonasRoot(final String jonasRoot) {
        this.jonasRoot = jonasRoot;
    }

    /**
     * Set JONAS_BASE value.
     * @param jonasBase the JONAS_BASE value
     */
    public void setJonasBase(final String jonasBase) {
        this.jonasBase = jonasBase;
    }

    /**
     * Set JAVA_HOME value.
     * @param javaHome the JAVA_HOME value
     */
    public void setJavaHome(final String javaHome) {
        this.javaHome = javaHome;
    }

    /**
     * Check the protocols value.
     * @return true if protocol list is valid, false otherwise.
     */
    private boolean isValidProtocolsParams() {
        return !getProtocolsList().equals("");
    }

    /**
     * Check the protocols value.
     * @return true if protocol list is valid, false otherwise.
     */
    private boolean isValidDbParams() {
        return !getDbName().equals("") && !getDbPort().equals("");
    }

    /**
     * Check the JMS value.
     * @return true if the jms parameters are valid, false otherwise.
     */
    private boolean isValidJmsParams() {
        return !getJmsPort().equals("");
    }

    /**
     * Check the HTTP value.
     * @return true if the http port is valid, false otherwise.
     */
    private boolean isValidHttpParams() {
        return !getHttpPort().equals("");
    }

    /**
     * Check the JDBC value.
     * @return true if the JDBC configuration is valid, false otherwise.
     */
    private boolean isValidJdbcConfigParams() {
        return true;
    }
}
